/*
 * Description
 *         HBASAMPLE.C - Implements a sample HBA API compliant vendor library
 *
 * Author
 *         Benjamin F. Kuo, Troika Networks, Inc. (benk@troikanetworks.com)
 *
 * Copyright (c) 2000 Troika Networks, Inc. All Rights Reserved.
 * 
 * Troika Networks, Inc., hereby grants a non-exclusive license under Troika Networks'
 * copyright to copy, modify and distribute this software for any purpose
 * and without fee, provided that the above copyright notice and the
 * following paragraphs appear on all copies.
 * 
 * Troika Networks, Inc.. makes no representation that this source code 
 * is correct or is an accurate representation of any standard.
 * 
 * IN NO EVENT SHALL TROIKA NETWORKS HAVE ANY LIABILITY FOR ANY DIRECT, INDIRECT, OR
 * SPECULATIVE DAMAGES, (INCLUDING WITHOUT LIMITING THE FORGOING,
 * CONSEQUENTIAL, INCIDENTAL AND SPECIAL DAMAGES) INCLUDING, BUT NOT 
 * LIMITED TO INFRINGEMENT, LOSS OF USE, BUSINESS INTERRUPTIONS, AND LOSS 
 * OF PROFITS, IRRESPECTIVE OF WHETHER TROIKA NETWORKS HAS ADVANCE NOTICE OF THE 
 * POSSIBILITY OF ANY SUCH DAMAGES.
 * 
 * TROIKA NETWORKS INC. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT 
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR 
 * A PARTICULAR PURPOSE AND NON-INFRINGEMENT.  THE SOFTWARE PROVIDED 
 * HEREUNDER IS ON AN "AS IS" BASIS AND TROIKA NETWORKS INC HAS NO 
 * OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR 
 * MODIFICATIONS.
 * 
 *************************************************************************** 
 */

 
 /* HBASAMPLE.c : Defines the entry point for the DLL application.
*/
#ifdef WIN32
#include <windows.h>
#endif

#include "hbaapi.h"
#include "hbasample.h"




/* Function prototypes */
HBA_UINT32 Sample_GetVersion();

HBA_STATUS Sample_LoadLibrary();

HBA_STATUS Sample_FreeLibrary();

HBA_UINT32 Sample_GetNumberOfAdapters();

HBA_STATUS Sample_GetAdapterName(HBA_UINT32 adapterindex, char *adaptername);

HBA_HANDLE Sample_OpenAdapter(
	char* adaptername
	);

void Sample_CloseAdapter(
	HBA_HANDLE handle
	);

HBA_STATUS Sample_GetAdapterAttributes(
	HBA_HANDLE handle, 
	HBA_ADAPTERATTRIBUTES *hbaattributes	
	);

HBA_STATUS Sample_GetAdapterPortAttributes(
	HBA_HANDLE handle, 
	HBA_UINT32 portindex,
	HBA_PORTATTRIBUTES *portattributes
	);

HBA_STATUS Sample_GetPortStatistics(
	HBA_HANDLE				handle,
	HBA_UINT32				portindex,
	HBA_PORTSTATISTICS			*portstatistics
	);

HBA_STATUS Sample_GetDiscoveredPortAttributes(
	HBA_HANDLE handle, 
	HBA_UINT32 portindex,
	HBA_UINT32 discoveredportindex,
	HBA_PORTATTRIBUTES *portattributes
	);

HBA_STATUS Sample_GetPortAttributesByWWN(
	HBA_HANDLE handle,
	HBA_WWN PortWWN,
	HBA_PORTATTRIBUTES *portattributes
	);

void Sample_RefreshInformation(HBA_HANDLE handle);

HBA_STATUS Sample_InitiateLIP();




/* Begin implementation */
#ifdef WIN32

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
#endif

/* This is the one exported function to register our library and
 Get entrypoints. */

HBASAMPLE_API HBA_STATUS HBA_RegisterLibrary( PHBA_ENTRYPOINTS entrypoints )
{
	entrypoints->LoadLibraryHandler = Sample_LoadLibrary;
	entrypoints->FreeLibraryHandler = Sample_FreeLibrary;
	entrypoints->GetVersionHandler = Sample_GetVersion;
	entrypoints->GetNumberOfAdaptersHandler = Sample_GetNumberOfAdapters;
	entrypoints->GetAdapterNameHandler = Sample_GetAdapterName;
	entrypoints->OpenAdapterHandler = Sample_OpenAdapter;
	entrypoints->CloseAdapterHandler = Sample_CloseAdapter;
	entrypoints->GetAdapterAttributesHandler = Sample_GetAdapterAttributes;
	entrypoints->GetAdapterPortAttributesHandler = Sample_GetAdapterPortAttributes;
	entrypoints->GetPortStatisticsHandler = Sample_GetPortStatistics;
	entrypoints->GetDiscoveredPortAttributesHandler = Sample_GetDiscoveredPortAttributes;
	entrypoints->GetPortAttributesByWWNHandler = Sample_GetPortAttributesByWWN;
	entrypoints->RefreshInformationHandler = Sample_RefreshInformation;

	return 0;
}

HBA_STATUS Sample_LoadLibrary()
{
	return HBA_STATUS_OK;
}

HBA_STATUS Sample_FreeLibrary()
{
	return HBA_STATUS_OK;
}

HBA_UINT32 Sample_GetVersion()
{
	return HBA_LIBVERSION;
}

HBA_UINT32 Sample_GetNumberOfAdapters() 
{
	return 1;
}

HBA_STATUS Sample_GetAdapterName(HBA_UINT32 adapterindex, char *adaptername)
{
	if (adapterindex > 1) {
		return HBA_STATUS_ERROR_ILLEGAL_INDEX;
	} else {
		strcpy(adaptername, "org.snia.sample-1");
		return HBA_STATUS_OK;
	}
}

HBA_HANDLE Sample_OpenAdapter(
	char* adaptername
	)
{
	HBA_HANDLE temp;
	temp = 0;
	if (strcmp(adaptername, "sample-adapter-1") == 0) {

	} 

	return temp;
}

void Sample_CloseAdapter(
	HBA_HANDLE handle
	)
{


}

HBA_STATUS Sample_GetAdapterAttributes(
	HBA_HANDLE handle, 
	PHBA_ADAPTERATTRIBUTES hbaattributes	
	)
{
	strcpy(hbaattributes->Manufacturer, "Emulex");
	strcpy(hbaattributes->SerialNumber, "A12345");
	strcpy(hbaattributes->Model, "QLA2200");
	strcpy(hbaattributes->ModelDescription, "Agilent TachLite");

	hbaattributes->NodeWWN.wwn[0] = 0x01;
	hbaattributes->NodeWWN.wwn[1] = 0x02;
	hbaattributes->NodeWWN.wwn[2] = 0x03;
	hbaattributes->NodeWWN.wwn[3] = 0x04;
	hbaattributes->NodeWWN.wwn[4] = 0x05;
	hbaattributes->NodeWWN.wwn[5] = 0x06;
	hbaattributes->NodeWWN.wwn[6] = 0x07;
	hbaattributes->NodeWWN.wwn[7] = 0x08;

	strcpy(hbaattributes->NodeSymbolicName, "HBA API Sample Driver");
	strcpy(hbaattributes->HardwareVersion, "Hardware Version");
	strcpy(hbaattributes->DriverVersion, "Driver Version");
	strcpy(hbaattributes->OptionROMVersion, "Option ROM Version");
	strcpy(hbaattributes->FirmwareVersion, "Firmware Version");
	hbaattributes->VendorSpecificID = 0;
	hbaattributes->NumberOfPorts = 1;
	return HBA_STATUS_OK;
}

HBA_STATUS Sample_GetAdapterPortAttributes(
	HBA_HANDLE handle, 
	HBA_UINT32 portindex,
	HBA_PORTATTRIBUTES *portattributes
	)
{
	if (portindex == 1) {
		/* WWN corresponding to 0001020304050708 */
		portattributes->NodeWWN.wwn[0] = 0x01;
		portattributes->NodeWWN.wwn[1] = 0x02;
		portattributes->NodeWWN.wwn[2] = 0x03;
		portattributes->NodeWWN.wwn[3] = 0x04;
		portattributes->NodeWWN.wwn[4] = 0x05;
		portattributes->NodeWWN.wwn[5] = 0x06;
		portattributes->NodeWWN.wwn[6] = 0x07;
		portattributes->NodeWWN.wwn[7] = 0x08;

		portattributes->PortWWN.wwn[0] = 0x01;
		portattributes->PortWWN.wwn[1] = 0x02;
		portattributes->PortWWN.wwn[2] = 0x03;
		portattributes->PortWWN.wwn[3] = 0x04;
		portattributes->PortWWN.wwn[4] = 0x05;
		portattributes->PortWWN.wwn[5] = 0x06;
		portattributes->PortWWN.wwn[6] = 0x07;
		portattributes->PortWWN.wwn[7] = 0x08;

		portattributes->PortFcId = 0x00;
		portattributes->PortType = HBA_PORTTYPE_UNKNOWN; 		
		portattributes->PortState = HBA_PORTSTATE_ONLINE;
		portattributes->PortSupportedClassofService = 0x00;
		portattributes->PortSupportedFc4Types.bits[0] = 0x00;
		portattributes->PortActiveFc4Types.bits[0] = 0x00;
		strcpy(portattributes->OSDeviceName, "DeviceName");
		portattributes->PortSpeed = HBA_PORTSPEED_1GBIT; 
		portattributes->NumberofDiscoveredPorts = 0;
		return HBA_STATUS_OK;
	} else {
		return HBA_STATUS_ERROR_ILLEGAL_INDEX;
	}
}

HBA_STATUS Sample_GetPortStatistics(
	HBA_HANDLE				handle,
	HBA_UINT32				portindex,
	HBA_PORTSTATISTICS			*portstatistics
	)
{
	portstatistics->TxFrames = 0;
	portstatistics->RxFrames = 0;
	portstatistics->TxWords = 0;
   	portstatistics->RxWords = 0;
	portstatistics->LIPCount = 0;
	portstatistics->NOSCount = 0;
	portstatistics->ErrorFrames = 0;
	portstatistics->DumpedFrames = 0;
	portstatistics->LinkFailureCount = 0;
	portstatistics->LossOfSyncCount = 0;
	portstatistics->LossOfSignalCount = 0;
	portstatistics->PrimitiveSeqProtocolErrCount = 0;
	portstatistics->InvalidTxWordCount = 0;
	portstatistics->InvalidCRCCount = 0;
	return HBA_STATUS_OK;
}

HBA_STATUS Sample_GetDiscoveredPortAttributes(
	HBA_HANDLE handle, 
	HBA_UINT32 portindex,
	HBA_UINT32 discoveredportindex,
	HBA_PORTATTRIBUTES *portattributes
	)
{
	/* since this is a virtual driver with no discovered ports 
	   always return an error */
	return HBA_STATUS_ERROR_ILLEGAL_INDEX;
}

HBA_STATUS Sample_GetPortAttributesByWWN(
	HBA_HANDLE handle,
	HBA_WWN PortWWN,
	HBA_PORTATTRIBUTES *portattributes
	)
{
	return HBA_STATUS_ERROR_NOT_SUPPORTED;
}

void Sample_RefreshInformation(HBA_HANDLE handle)
{

}

HBA_STATUS Sample_InitiateLIP()
{
	return HBA_STATUS_ERROR_NOT_SUPPORTED;
}
